(* Content-type: application/mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 6.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       145,          7]
NotebookDataLength[     70575,       1918]
NotebookOptionsPosition[     64620,       1714]
NotebookOutlinePosition[     65041,       1730]
CellTagsIndexPosition[     64998,       1727]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell["Mathematica workshop - worksheet 2", "Title",
 CellChangeTimes->{{3.400477455849*^9, 3.400477457334*^9}, 3.402663754282*^9, 
   3.4035292407130003`*^9}],

Cell["Organising code", "Subtitle",
 CellChangeTimes->{{3.4004772457130003`*^9, 3.400477249104*^9}, {
  3.4004774883900003`*^9, 3.400477495001*^9}}],

Cell[TextData[{
 "Please don't send people copies of this worksheet. Instead, direct them to\n\
",
 ButtonBox["http://j-star.org/mathematica_course.html",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://j-star.org/mathematica_course.html"], None}],
 "\n where the latest version will always be available.\n \[Copyright] Jony \
Hudson, Imperial College London, 2006-2011"
}], "Text",
 CellChangeTimes->{{3.399981956925*^9, 3.399982088848*^9}, {
   3.4035292541429996`*^9, 3.4035292548780003`*^9}, 3.416690493255*^9, {
   3.5250731862785378`*^9, 3.525073188548541*^9}},
 TextAlignment->Center,
 Background->RGBColor[0.7098039215686275, 0.8, 0.7137254901960784]],

Cell[TextData[{
 "In the last worksheet we looked at some elements of ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " programming that come up a lot, and are a bit different from what you \
might be used to.\n\nIn this worksheet we'll look a little more \
systematically at one of the key aspects of programming in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " - using functions to abstract operations. We'll do this in the context of \
basic data analysis tasks - the sort of thing that come up in the lab all the \
time.  We'll see how the ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " style (more generally, the functional programming style) is to build \
complicated systems from the bottom-up using simple ingredients. [People \
often argue that this bottom-up design is one of the key features of working \
with a functional language, as opposed to the top-down design more common \
with imperative languages. I think they just like to argue about stuff, but \
hey ...] "
}], "Text",
 CellChangeTimes->{{3.4004772527530003`*^9, 3.400477425457*^9}}],

Cell[CellGroupData[{

Cell["Plotting and fitting", "Section"],

Cell[TextData[{
 "Let's get warmed up by doing some simple least squares fitting. I'll start \
with an example and then move onto some exercises.\n\nHere are some data I \
took while characterising the LIF molecule detector on the electron edm \
experiment. The data are a record of a photomultiplier's (PMT) output in \
current- and counting-modes for various levels of illumination. The data are \
pairs of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{
     RowBox[{"amplifier", " ", "output", " ", 
      RowBox[{"(", "mV", ")"}]}], ",", " ", 
     RowBox[{"count", " ", "rate", " ", 
      RowBox[{"(", "kHz", ")"}]}]}], "}"}], TraditionalForm]]],
 "."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"data", " ", "=", " ", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"15", ",", "123"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"17", ",", "130"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"31", ",", "215"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"36", ",", "253"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"47", ",", "297"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"58", ",", "388"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"65", ",", "412"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"82", ",", "510"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"160", ",", "910"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"250", ",", "1320"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"500", ",", "2260"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"870", ",", "3150"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"1120", ",", "3650"}], "}"}]}], "}"}]}], ";"}]], "Input"],

Cell["Let's take a look at the data.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"dataPlot", "=", 
  RowBox[{"ListPlot", "[", 
   RowBox[{"data", ",", 
    RowBox[{"AxesLabel", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{"\"\<amp output (mV)\>\"", ",", "\"\<count rate (kHz)\>\""}], 
      "}"}]}], ",", 
    RowBox[{"PlotStyle", "\[Rule]", 
     RowBox[{"PointSize", "[", "0.02", "]"}]}]}], "]"}]}]], "Input"],

Cell[BoxData[
 GraphicsBox[{{}, 
   {Hue[0.67, 0.6, 0.6], PointSize[0.02], 
    PointBox[{{15., 123.}, {17., 130.}, {31., 215.}, {36., 253.}, {47., 
     297.}, {58., 388.}, {65., 412.}, {82., 510.}, {160., 910.}, {250., 
     1320.}, {500., 2260.}, {870., 3150.}, {1120., 3650.}}]}, {}},
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  Axes->True,
  AxesLabel->{
    FormBox["\"amp output (mV)\"", TraditionalForm], 
    FormBox["\"count rate (kHz)\"", TraditionalForm]},
  AxesOrigin->{0, 0},
  PlotRange->{{0., 1120.}, {0., 3650.}},
  PlotRangeClipping->True,
  PlotRangePadding->{
    Scaled[0.02], 
    Scaled[0.02]}]], "Output",
 CellChangeTimes->{3.4004775335290003`*^9, 3.40058425571*^9, 
  3.4005849760369997`*^9, 3.403529415168*^9, 3.416688774025*^9, 
  3.4166890348719997`*^9, 3.443183632477192*^9, 3.4431844513030663`*^9, 
  3.524468228138129*^9, 3.5250732463586216`*^9, 3.5250968047792177`*^9}]
}, Open  ]],

Cell["\<\
We can clearly see the counting mode response becoming non-linear due to \
pulse pile-up in the counting electronics (don't worry if you don't really \
know anything about PMTs, or indeed experimental physics - it doesn't matter \
much for the examples). 

The Mathematica command FindFit can be used to fit a model to data. Let's \
model the data as a saturating exponential with an offset. I've given the \
fitter some reasonable guesses for the parameters.\
\>", "Text",
 CellChangeTimes->{{3.403529313953*^9, 3.40352932508*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"model", "=", 
   RowBox[{"a", "+", 
    RowBox[{"b", 
     RowBox[{"(", 
      RowBox[{"1", "-", " ", 
       SuperscriptBox["\[ExponentialE]", 
        RowBox[{
         RowBox[{"-", "c"}], " ", "x"}]]}], ")"}]}]}]}], ";"}]], "Input",
 CellChangeTimes->{{3.5250967923792*^9, 3.525096795089204*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"fit", "=", 
  RowBox[{"FindFit", "[", 
   RowBox[{"data", ",", "model", ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"a", ",", "100"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"b", ",", "5000"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"c", ",", 
        SuperscriptBox["10", 
         RowBox[{"-", "3"}]]}], "}"}]}], "}"}], ",", "x"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.525096806039219*^9, 3.5250968097892246`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"a", "\[Rule]", "39.81124300566447`"}], ",", 
   RowBox[{"b", "\[Rule]", "4757.654641265243`"}], ",", 
   RowBox[{"c", "\[Rule]", "0.0012491568239159814`"}]}], "}"}]], "Output",
 CellChangeTimes->{3.400477556686*^9, 3.400584256401*^9, 3.400584976759*^9, 
  3.4035294169440002`*^9, 3.416688775493*^9, 3.416689036119*^9, 
  3.4431836638743315`*^9, 3.4431844521691527`*^9, 3.5244682297481318`*^9, 
  3.525073255488634*^9, 3.525073757149337*^9, 3.525096810799226*^9}]
}, Open  ]],

Cell[TextData[{
 "The best fit is returned as a set of rules.We can use these rules to \
transform the general model above to the best-fit model (using ",
 Cell[BoxData[
  FormBox["/.", TraditionalForm]]],
 "- look it up if you haven't already done so). "
}], "Text",
 CellChangeTimes->{{3.400477581817*^9, 3.400477592157*^9}, 
   3.5250732751786623`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"model", "/.", "fit"}]], "Input",
 CellChangeTimes->{{3.5250732813786707`*^9, 3.5250732829686728`*^9}}],

Cell[BoxData[
 RowBox[{"39.81124300566447`", "\[VeryThinSpace]", "+", 
  RowBox[{"4757.654641265243`", " ", 
   RowBox[{"(", 
    RowBox[{"1", "-", 
     SuperscriptBox["\[ExponentialE]", 
      RowBox[{
       RowBox[{"-", "0.0012491568239159814`"}], " ", "x"}]]}], 
    ")"}]}]}]], "Output",
 CellChangeTimes->{3.5250732834086733`*^9, 3.5250737576393375`*^9, 
  3.5250968130392294`*^9}]
}, Open  ]],

Cell["\<\
We see the replacement of the parameters with their fitted values. Let\
\[CloseCurlyQuote]s plot the data and fit on the same graph.\
\>", "Text",
 CellChangeTimes->{{3.5250732793986683`*^9, 3.5250733045587034`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Show", "[", "\[IndentingNewLine]", 
  RowBox[{"dataPlot", ",", "\[IndentingNewLine]", 
   RowBox[{"Plot", "[", 
    RowBox[{
     RowBox[{"model", "/.", "fit"}], ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", "0", ",", "1200"}], "}"}], ",", 
     RowBox[{"PlotStyle", "\[Rule]", "Red"}]}], "]"}]}], 
  "\[IndentingNewLine]", "]"}]], "Input",
 CellChangeTimes->{{3.400477567153*^9, 3.4004775676949997`*^9}}],

Cell[BoxData[
 GraphicsBox[{{{}, 
    {Hue[0.67, 0.6, 0.6], PointSize[0.02], 
     PointBox[{{15., 123.}, {17., 130.}, {31., 215.}, {36., 253.}, {47., 
      297.}, {58., 388.}, {65., 412.}, {82., 510.}, {160., 910.}, {250., 
      1320.}, {500., 2260.}, {870., 3150.}, {1120., 3650.}}]}, {}}, {{}, {}, 
    {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwVxXk41HkAB2B0CLP0RIjMspu1lKN2tjUd+nhC9My2ISKiUXpcoUQ8M1nL
DOuYMb6mlHuM2/y6UDYZqqdaPZvKOrJR24HBOEqOaeuxu3+8z2sREuMVqqGm
pvbjf/6/0ffT3CSlclKZcMTPR3cj835Qv6ewfxej6pfjd5dcIPorlUMXju5S
vf7SI8vIDeK34kfLhQu7nPylPzW7ueP8WMn8gkATdwY2dqVIWTCK+ir7ncAI
LO2kpMQGLxykjvlMCazg18IJq/UKwMIj/k2l4Ad0tzi5MB2Po9Hg6Rfp2e7Q
z4lF7abTiN3IeNK23Qsx0XURRWwuDPcsiDv1/WDv+OBxm3EqZtuMIioignDb
tKB0gpEOfjDduMAmBNp24yyXyUzIBMFpu4NDcbIjpJvWJ4BpUmDUIi8M4mHG
7Zn7ItTUPX+UfjoShSZzI/nxBF0ZmTc40mgMmqiv5G4TI6t3kGbZFQsvsSXz
bM85PLnXzV29+xTWRBZVPzhUgLe0vx0u98fhGiuAVjB3ARkJdpJtnvFgTTFb
2eGFWNgq9kkaTwAj/rI5Y6YIOn397ju4idCb7NNSHi3BpYLAxYn3SfAUjWns
bCoF/crixL5MDubndF01dcphNtFbVLr8LKINE+qtG8oxzn2YKylLBuEtvgjz
loBK2McWr0vB1FWDI1GzEtjimV9XbwrOnUi0YfIqoN984v0ryS/wfWcbylkv
hXPr1+ORR1Jhoset966W4vYVd3VH0zSQu4OlWdaVqPFMujk0nAbeQKy8vLES
vWkPn6oqeIiML0sqcqhCBNsqezqGD6VOQMxcYxU87WaPWjHToVr5VOeNQzV+
znlZ2PshHVOu0jalpBoG5/vjDndkoNf+n5Ru4xrMOMzUByT/CmVnkCKOVwN6
6xn1YsdMLDgvu6MYr0HjjPbHZo0saLIVI3d9a5F7Q77frT0LY3X96cEttei2
Tva4kp0N59klp8/0OgTaBpWL9uZAyV5VNsypg6M5Zdi4QoB41dZin546cOSa
9pefCGB++FVVyOZ6rJhKrbIQCuGi9ZhWzK/H/O9a8hMBuVD/7BS9/kU9ijuX
Pp5dI4L8GjPcfUsD2AvabimxIghOmTw7ldyAgY+mITF9IujlSgxzuhow57aT
uvh9Hjw2WEfoGclwU9tw1TDJg1aYpp5hoAxbKj98Z7aYh8SSB/63KmXYl/5G
KPIlqOi2KOOOyLBnr5C11Y9AtyKnp2RUBugytQf9CdSevX4jV8iw+byQb3WY
APxQkdqEDAbVTK78KEGz+X6L1GkZnt/LDVfGEmzRNbFKXpQhbMUOF/dsAu+X
AcP+2hTYnSPqUzkEf7YPsRN0KBwS5LXnCwkuMsZbCI0Ca+3otpd5BArx9fmH
uhTsLcnmhAsEby1qrjL1Kcy7KOiVVQQ3+hnWq9dTmF6VP+hRQ0BiLu23MaOg
+GNn4XQtQZCjurcLncLAgfy122UEByddFWfMKdw65kTrvkYgberxGdpAofnb
sc4zTQTs2vID85YULinzM8yuE8QORdnpWVEoPz2mEf4bAT9jXRysKVx0FHfo
thJMtyx74WdDgXxySm66RRBXqb7h5EYKWR1j2w/JCWq/MXbN3EQhjSdWLbUT
8ExZzhJbCv8Cq7tQpg==
      "]]}}},
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  Axes->True,
  AxesLabel->{
    FormBox["\"amp output (mV)\"", TraditionalForm], 
    FormBox["\"count rate (kHz)\"", TraditionalForm]},
  AxesOrigin->{0, 0},
  PlotRange->{{0., 1120.}, {0., 3650.}},
  PlotRangeClipping->True,
  PlotRangePadding->{
    Scaled[0.02], 
    Scaled[0.02]}]], "Output",
 CellChangeTimes->{3.4004775698310003`*^9, 3.400584256946*^9, 
  3.400584977502*^9, 3.403529417802*^9, 3.416688776112*^9, 3.416689036796*^9, 
  3.4431836767236156`*^9, 3.4431844526351995`*^9, 3.524468230828133*^9, 
  3.5250737583593388`*^9, 3.525096814439231*^9}]
}, Open  ]],

Cell["\<\
Looks ok. I took the data to find the calibration between count rate and \
voltage output in the PMT's linear region. How do we extract it? It's just \
the derivative of the fitted model at zero count rate.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"D", "[", 
   RowBox[{
    RowBox[{"model", "/.", "fit"}], ",", "x"}], "]"}], "/.", 
  RowBox[{"x", "\[Rule]", "0"}]}]], "Input"],

Cell[BoxData["5.943056760972019`"], "Output",
 CellChangeTimes->{3.400584257061*^9, 3.400584978217*^9, 3.403529418642*^9, 
  3.416688776705*^9, 3.416689037509*^9, 3.4431836899099345`*^9, 
  3.4431844529842343`*^9, 3.524468232088135*^9, 3.5250737589693394`*^9, 
  3.5250968156792326`*^9}]
}, Open  ]],

Cell["\<\
Each mV of output is about 6000 photons/second. Note that we've got no idea \
what the uncertainty in this number is. Let's ignore this for the moment - \
we'll come back to it later.\
\>", "Text"],

Cell[CellGroupData[{

Cell["Exercises", "Subsection"],

Cell["\<\
The code above is sufficient to fit a model to some data. In this section \
we'll start to abstract out the key operations. This will provide a \
foundation on which to perform more sophisticated data analysis tasks.\
\>", "Text"],

Cell[CellGroupData[{

Cell["Plotting the fitted data", "Subsubsection"],

Cell[TextData[{
 "Write a function that takes a list of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"x", ",", "y"}], "}"}], TraditionalForm]]],
 " pairs and a fitted model and plots the data and fit on the same graph. The \
fit plot should run over the range of ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " in the data list. Test your function by evaluating it with the data, model \
and fit from the above example (you'll need to run the cells above first).\n",
 StyleBox["Hint: ",
  FontWeight->"Bold"],
 "this is quite a complicated function so you might find a Module is useful.\n\
",
 StyleBox["Hint 2:",
  FontWeight->"Bold"],
 " don't worry about axes labels and other bells and whistles for the moment. \
Unless you want to, of course!\n",
 StyleBox["Hint 3:",
  FontWeight->"Bold"],
 " you might find the functions Min and Max useful.\n",
 StyleBox["Hint 4",
  FontWeight->"Bold"],
 ": re-read the question! Lots of people misundertand it and answer a \
different question."
}], "Text",
 CellChangeTimes->{{3.400477652876*^9, 3.4004776709110003`*^9}, {
  3.525073378758807*^9, 3.5250734125388546`*^9}}]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Solution", "Subsubtitle"],

Cell[CellGroupData[{

Cell["Testing with another fit", "Subsubsection"],

Cell[BoxData[
 RowBox[{
  RowBox[{"sinData", "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "4.8"}], ",", "542"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "4.7"}], ",", "489"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "4.5"}], ",", "407"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "4.4"}], ",", "331"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "4.2"}], ",", "247"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "4.0"}], ",", "191"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "3.9"}], ",", "170"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "3.7"}], ",", "200"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "3.5"}], ",", "256"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "3.4"}], ",", "337"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "3.2"}], ",", "410"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "3.1"}], ",", "476"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "2.9"}], ",", "498"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "2.7"}], ",", "486"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "2.6"}], ",", "447"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "2.4"}], ",", "346"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "2.3"}], ",", "266"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "2.1"}], ",", "199"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "1.9"}], ",", "164"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "1.8"}], ",", "171"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "1.6"}], ",", "226"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "1.5"}], ",", "303"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "1.3"}], ",", "397"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "1.1"}], ",", "459"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "0.97"}], ",", "493"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "0.81"}], ",", "486"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "0.65"}], ",", "429"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "0.48"}], ",", "358"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "0.32"}], ",", "291"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "0.16"}], ",", "220"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", "172"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0.16", ",", "174"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0.32", ",", "213"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0.48", ",", "280"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0.65", ",", "339"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0.81", ",", "416"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0.97", ",", "472"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"1.1", ",", "477"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"1.3", ",", "453"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"1.5", ",", "387"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"1.6", ",", "322"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"1.8", ",", "248"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"1.9", ",", "192"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"2.1", ",", "169"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"2.3", ",", "180"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"2.4", ",", "228"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"2.6", ",", "308"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"2.7", ",", "380"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"2.9", ",", "460"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"3.1", ",", "506"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"3.2", ",", "515"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"3.4", ",", "449"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"3.5", ",", "382"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"3.7", ",", "280"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"3.9", ",", "202"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"4.0", ",", "172"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"4.2", ",", "171"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"4.4", ",", "209"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"4.5", ",", "278"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"4.7", ",", "362"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"4.8", ",", "423"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"5.0", ",", "465"}], "}"}]}], "}"}]}], ";"}]], "Input"],

Cell[TextData[{
 "The data above make up a scan from the electron edm molecular spin \
interferometer. The data are pairs of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{
     RowBox[{"applied", " ", "magnetic", " ", "field"}], " ", ",", " ", 
     RowBox[{
      RowBox[{"population", " ", "in", " ", "F"}], "=", 
      RowBox[{"1", " ", "hyperfine", " ", "level"}]}]}], "}"}], 
   TraditionalForm]]],
 ". They show the phase between two near-degenerate magnetic sublevels of the \
YbF molecule evolving as a small magnetic field is scanned. Fit the data to a \
",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["Sin", "2"], TraditionalForm]]],
 " model with an offset and scale parameter in both ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 ".\n",
 StyleBox["Hint:",
  FontWeight->"Bold"],
 " you'll need to provide some reasonable initial guesses to get the fit to \
converge.\n",
 StyleBox["Note:",
  FontWeight->"Bold"],
 " I'm sure you've got the point by now, but your solution shouldn't directly \
call Plot, ListPlot etc."
}], "Text"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Solution", "Subsubtitle"],

Cell[CellGroupData[{

Cell["Importing", "Section"],

Cell[TextData[{
 "Great. So we can fit data to models and plot the resulting fits. Next we'll \
go through another simple task that comes up frequently - importing and \
fitting data. This stuff is all rather pedestrian, but it'll come together to \
make something interesting in the end!\n\nThe ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " function for importing structured data is, perhaps unsurprisingly, named \
Import. We'll use it here to import some textual data and fit it."
}], "Text"],

Cell[CellGroupData[{

Cell["Exercises", "Subsection"],

Cell[TextData[{
 "This command sets ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "'s working directory to the data subdirectory of the directory that this \
notebook is in. [As long as you didn't move the notebook out of the unzipped \
archive then this should work.] [And assuming you're using Windows! If you're \
using another OS you might need to tweak it a little.]"
}], "Text",
 CellChangeTimes->{{3.4035295176619997`*^9, 3.403529545993*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"SetDirectory", "[", "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"DirectoryName", "[", 
     RowBox[{"ToFileName", "[", 
      RowBox[{"\"\<FileName\>\"", "/.", 
       RowBox[{"NotebookInformation", "[", 
        RowBox[{"SelectedNotebook", "[", "]"}], "]"}]}], "]"}], "]"}], "<>", 
    "\"\<data\\\\\>\""}], "\[IndentingNewLine]", "]"}], ";"}]], "Input"],

Cell[CellGroupData[{

Cell["Import, fit, plot", "Subsubsection"],

Cell[TextData[{
 "Import the data file \"first.txt\". The file has two columns of numbers \
delimited by whitespace. You should use the import mode \"TABLE\" to import \
this data. Write a function to fit and plot data. Use it to fit a Lorentzian \
(with background, amplitude, centre and width) to the data and plot the \
resultant fit on the same graph as the data. Return both the fit and the \
plot.\n",
 StyleBox["Hint:",
  FontWeight->"Bold"],
 " this might sound intimidatingly difficult, but it's nothing more that \
you've done already. Build the function step by step.\n",
 StyleBox["Hint 2:",
  FontWeight->"Bold"],
 " in case you don't remember, a Lorentzian is something like ",
 Cell[BoxData[
  FormBox[
   FractionBox["1", 
    RowBox[{"1", "+", 
     SuperscriptBox[
      RowBox[{"(", 
       FractionBox[
        RowBox[{"x", "-", "c"}], "\[CapitalGamma]"], ")"}], "2"]}]], 
   TraditionalForm]]],
 ".\n",
 StyleBox["Hint 3:",
  FontWeight->"Bold"],
 " the last expression in a Module will be returned as the module's value \
when it's evaluated.\n",
 StyleBox["Hint 4:",
  FontWeight->"Bold"],
 " You're returning two things. Why not return them as a pair of rules ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{
     RowBox[{"fit", "\[Rule]", "someStuff"}], ",", 
     RowBox[{"plot", "\[Rule]", "thePlot"}]}], "}"}], TraditionalForm]]],
 ".\n",
 StyleBox["Hint 5:",
  FontWeight->"Bold"],
 " I often use strings as the first part of a rule, as it avoids clashes with \
previously defined symbols.\n"
}], "Text",
 CellChangeTimes->{{3.400584129692*^9, 3.400584218304*^9}, {3.400584290231*^9,
   3.400584317892*^9}, {3.400584407406*^9, 3.400584427601*^9}, {
  3.403529614574*^9, 3.403529632936*^9}, {3.403530174288*^9, 
  3.403530174435*^9}, {3.4035302098389997`*^9, 3.4035302435299997`*^9}}]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Solution", "Subsubtitle"],

Cell[CellGroupData[{

Cell["Automating", "Section"],

Cell[TextData[{
 "Now we'll use our new functions to carry out a more complicated data \
analysis task. But before we get on to that, let's think a little about what \
we're actually doing when we write functions. The code we've written so far \
is pretty trivial in the grand scheme of things - we've only defined two \
functions - but already we can see what's happening. Effectively, by making a \
function that fits and plots the resulting fit, we've added that capability \
to ",
 StyleBox["Mathematica. ",
  FontSlant->"Italic"],
 "It's almost like we're building a better ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ". In this next section we'll write code that automatically loads and fits \
hundreds of data files and looks for trends in the fit parameters. This code \
will use our functions defined above so we won't have to worry about how to \
do the fitting when we write it. We've already worried about that, solved it, \
and encapsulated it which will allow us to concentrate on just what's new. If \
we write the code for the next section carefully then we could further write \
code that takes hundreds of sets of hundreds of data files, and extracts \
trends of trends (which is starting to sound like the edm data analysis \
system!).\n\nThis layering up of functionality is what the arguing computer \
scientists I mentioned earlier call bottom-up design. It's a very powerful \
way to work. You extend the language layer by layer so that it becomes a more \
and more powerful language to solve your problem. There are a couple of key \
habits you need to get into to be able to program this way. First is to \
abstract out any emerging structure as early as you can: never be too lazy to \
write a function; it takes a little effort but you kind of get an exponential \
payback from writing a few good functions so it's almost always worth it. \
Second, if something feels wrong, fix it right away. For example, if you \
think a function should really be taking an extra parameter, or should be \
split into two functions then get into the habit of doing it (and modifying \
the rest of your code to reflect the change) before you do anything else. I \
find this quite a hard habit to keep, as it seems like a lot of work, but I \
think it's essential. You can't build a solid program on a dubious foundation \
- if you don't fix problems early you'll find that you'll get to a point \
where you're stuck and you can no longer face going back and fixing it \
because the problem is so deeply embedded in your code. When that happens to \
me I usually start writing again and resolve not to be so lazy next time!\n\n\
It's quite difficult to work this way at first, especially when you find that \
minor problems of not knowing the language well are making things harder. But \
I'd urge you to keep these ideas in mind when you're writing your own code. \
It's worth the effort.\n\nRight. That's enough preaching for one workbook. \
Back to the exercises."
}], "Text"],

Cell[CellGroupData[{

Cell["Exercises", "Subsection"],

Cell["\<\
There are 376 data files in the data directory named \"D1.txt\" through \
\"D376.txt\". These files each contain a scan over a Lorentzian line - the \
data is in the same format as above. In the following exercises we'll fit all \
of these Lorentzians and look for a pattern in the fit parameters.\
\>", "Text"],

Cell[CellGroupData[{

Cell["Importing and fitting a scan", "Subsubsection"],

Cell[TextData[{
 "First we should write a function to import and fit a scan to a Lorentzian. \
We've done almost all of the work already - here we'll just tie up a few \
ends. Your function should take the scan filename as a parameter.\n\nTest the \
new function on \"D75.txt\". \n\n",
 StyleBox["Hint:",
  FontWeight->"Bold"],
 " to keep things from getting too complicated you can assume that giving the \
fitter a starting parameter of 2 for the Lorentzian centre, and 0.5 for the \
Lorentzian width is all that is needed to make it work. You can assume that \
this will be true for all of the scans."
}], "Text",
 CellChangeTimes->{{3.400584597196*^9, 3.400584611616*^9}, {3.400584767249*^9,
   3.400584786032*^9}, {3.403530278645*^9, 3.4035302825150003`*^9}}]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Solution", "Subsubtitle"],

Cell[CellGroupData[{

Cell["Generating a list of file names", "Subsubsection"],

Cell[TextData[{
 "Write a pure function that takes an integer, ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ", and returns the string \"D",
 StyleBox["n",
  FontSlant->"Italic"],
 ".txt\". Map this function over the range 1 to 376 to generate a list of \
filenames to analyse.\n",
 StyleBox["Hint: ",
  FontWeight->"Bold"],
 "StringJoin (infix shorthand",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", "<>"}], TraditionalForm]]],
 ") and ToString might be useful here."
}], "Text"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Solution", "Subsubtitle"],

Cell[CellGroupData[{

Cell["Fitting the data", "Subsubsection"],

Cell[TextData[{
 "Now you have a function that takes a data filename and a list of filenames. \
Map the importing/fitting function over the list of files to generate a list \
of fits.\n",
 StyleBox["Hint:",
  FontWeight->"Bold"],
 " remember that a semicolon will stop ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " from printing out all of the results.\n",
 StyleBox["Note:",
  FontWeight->"Bold"],
 " this should take around ten to twenty seconds to run. If it takes much \
more or less then start to worry."
}], "Text",
 CellChangeTimes->{{3.400584799451*^9, 3.400584814417*^9}, {3.403530365266*^9,
   3.403530368502*^9}, {3.403530407949*^9, 3.403530409712*^9}, {
  3.443184067368677*^9, 3.4431840705239925`*^9}}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Solution", "Subsubtitle"],

Cell[CellGroupData[{

Cell["Looking for a pattern", "Subsubsection"],

Cell[TextData[{
 "Now we can look for a pattern in the fit parameters. Let's assume - this \
example is rather contrived - that the secrets of the universe will be \
revealed if we plot the fitted background on the x-axis and the fitted \
Lorentzian amplitude on the y-axis. ListPlot these fit parameters as just \
described.\n",
 StyleBox["Hint:",
  FontWeight->"Bold"],
 " remember the fits are returned as lists of rules. You can use the ",
 Cell[BoxData[
  FormBox["/.", TraditionalForm]]],
 " operator to effect transformations with these rules."
}], "Text"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Solution", "Subsubtitle"],

Cell[CellGroupData[{

Cell["Uncertainty estimates", "Section"],

Cell[TextData[{
 "In the above examples we've not considered what the uncertainties in our \
fitted parameters are. These uncertainties are of course essential if we are \
to interpret our (or others\[CloseCurlyQuote]) results correctly.\n\n[If you \
don't know much about statistics then you'll probably find the following \
section rather hard to follow.  I'd strongly suggest spending a little time \
reading up on it though because it's something you need to know about. A \
nice, clear introduction can be found in the chapters on fitting data in \
Numerical Recipes. It's well worth having a decent understanding of \
statistics if you want to report results in your papers that are, well, you \
know, right. I'll try to refrain from ranting, but you might be surprised by \
how many papers you'll read where the authors make a hash of basic \
statistics. Theorists are not exempt from needing this understanding - \
presuming that you might want to tie your work to actual results!]\n\nThe \
function FindFit does not offer errors in the fit parameters, but two similar \
functions LinearModelFit and NonlinearModelFit (note the strange \
capitalisation) do. Before we investigate these functions let's quickly \
revise what kinds of confidence estimates we should expect to be able to get. \
For a linear fit (note well that linear regression applies to models that are \
linear in the parameters, not the variables) with errors on the data points, \
the errors on the fitted parameters are uniquely determined without further \
assumption. This is the best, and hopefully most common case. A linear fit \
with no errors on the points can still yield errors on the fitted parameters, \
but only under the assumptions that the data have similar error bars, and \
we've correctly chosen the form of the model underlying the data - note that \
this can be a very dangerous assumption. [Amazingly, this point is not \
understood by a number of reasonably eminent physicists I've met. Make sure \
that you do understand it!] For non-linear fits it gets a bit more tricky - \
one of the standard techniques is to find confidence ellipsoids by finding \
the excursions in parameter space from the optimum that change ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["\[Chi]", "2"], TraditionalForm]]],
 " by a certain amount. Again this only works without further assumption if \
you have error bars on the data points. \n\nSo how do ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "'s regression functions relate to these techniques? We'll take a look at \
that in the following exercises and examples."
}], "Text",
 CellChangeTimes->{{3.4035304426689997`*^9, 3.403530460023*^9}, {
  3.443188693637257*^9, 3.4431887026391573`*^9}, {3.443189250664954*^9, 
  3.443189268342722*^9}, {3.5250738504894676`*^9, 3.5250738583194785`*^9}, {
  3.5250739311795807`*^9, 3.525073999939677*^9}}],

Cell[CellGroupData[{

Cell["Fitting with LinearModelFit", "Subsubsection",
 CellChangeTimes->{{3.443184578178753*^9, 3.4431845801769524`*^9}}],

Cell["\<\
LinearModelFit is used much like FindFit. The differences are: you give it a \
list of basis functions instead of a model; and it returns much more \
information about the fit. Let's use it to fit a straight line - I'll make up \
some data with slope 3, offset 5, and Gaussian y-noise with a standard \
deviation of 4 on the points. I'll use ErrorListPlot to plot the data with \
error bars.\
\>", "Text",
 CellChangeTimes->{{3.416688989803*^9, 3.416688991826*^9}, 
   3.443184583381273*^9}],

Cell[BoxData[
 RowBox[{"Needs", "[", "\"\<ErrorBarPlots`\>\"", "]"}]], "Input",
 CellChangeTimes->{{3.416688979677*^9, 3.416688987*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"linData", "=", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"#1", ",", 
        RowBox[{
         RowBox[{"3", " ", "#1"}], "+", "5", "+", 
         RowBox[{"Random", "[", 
          RowBox[{"NormalDistribution", "[", 
           RowBox[{"0", ",", "4"}], "]"}], "]"}]}]}], "}"}], "&"}], ")"}], "/@", 
    RowBox[{"Range", "[", "30", "]"}]}]}], ";"}], "\n", 
 RowBox[{"ErrorListPlot", "[", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"#1", ",", 
        RowBox[{"ErrorBar", "[", "4", "]"}]}], "}"}], "&"}], ")"}], "/@", 
    "linData"}], "}"}], "]"}]}], "Input",
 CellChangeTimes->{{3.400584872641*^9, 3.4005848750889997`*^9}, {
  3.40058491027*^9, 3.400584930517*^9}, {3.40058520371*^9, 3.400585207299*^9}}
  ],

Cell[BoxData[
 GraphicsBox[{{}, 
   {Hue[0.67, 0.6, 0.6], PointBox[CompressedData["
1:eJxTTMoPSmViYGCQA2IQDQEf7JlaPfYVKog7QAUcnmybOS/8mTaUz+GgL7gx
cY2APpQv4PBKrNbTczeML+Lw7Zu+4SNfcyhfwuFgzmPtCeEWUL6Mg7WV2S2v
fjsoX8FhQi2vXVqDE5Sv5NBSohAfccgWyldxuBnBnnv2twOUr+awd2LZKoaZ
MPUaDocPvlHxdIPxtRwWFK+/2/TcGcrXccivUEj9auUO5es5HFJ3m/ZY2wvK
N3Bo4Jw2u5UpAMo3dPBSyPEyi/aB8o0c3jNdceZc5wflGzv0K7wzZRGDqTdx
yLeZdm7ddBjf1EHCtaP73IZAKN/MYVmQ1ayb3UFQvrlD9veNu415gqF8C4dg
K/fmV1dh6i0dlNdyiLVlh0D5Vg6r18/o2bkGxrd24DvlxnlGCca3cbBa+ELU
liMMyrd1CJz20PQqI4xv53DsDbORPm+kAwDNxXG5
     "]], {{LineBox[{{1., 9.781683896232837}, {1., 1.7816838962328365`}}], 
      LineBox[{Offset[{1.5, 0}, {1., 9.781683896232837}], 
        Offset[{-1.5, 0}, {1., 9.781683896232837}]}], 
      LineBox[{Offset[{1.5, 0}, {1., 1.7816838962328365`}], 
        Offset[{-1.5, 0}, {1., 1.7816838962328365`}]}]}, {
      LineBox[{{2., 17.94988723398587}, {2., 9.94988723398587}}], 
      LineBox[{Offset[{1.5, 0}, {2., 17.94988723398587}], 
        Offset[{-1.5, 0}, {2., 17.94988723398587}]}], 
      LineBox[{Offset[{1.5, 0}, {2., 9.94988723398587}], 
        Offset[{-1.5, 0}, {2., 9.94988723398587}]}]}, {
      LineBox[{{3., 19.532565167297975`}, {3., 11.532565167297973`}}], 
      LineBox[{Offset[{1.5, 0}, {3., 19.532565167297975`}], 
        Offset[{-1.5, 0}, {3., 19.532565167297975`}]}], 
      LineBox[{Offset[{1.5, 0}, {3., 11.532565167297973`}], 
        Offset[{-1.5, 0}, {3., 11.532565167297973`}]}]}, {
      LineBox[{{4., 19.86579351093265}, {4., 11.865793510932651`}}], 
      LineBox[{Offset[{1.5, 0}, {4., 19.86579351093265}], 
        Offset[{-1.5, 0}, {4., 19.86579351093265}]}], 
      LineBox[{Offset[{1.5, 0}, {4., 11.865793510932651`}], 
        Offset[{-1.5, 0}, {4., 11.865793510932651`}]}]}, {
      LineBox[{{5., 27.30423266812337}, {5., 19.30423266812337}}], 
      LineBox[{Offset[{1.5, 0}, {5., 27.30423266812337}], 
        Offset[{-1.5, 0}, {5., 27.30423266812337}]}], 
      LineBox[{Offset[{1.5, 0}, {5., 19.30423266812337}], 
        Offset[{-1.5, 0}, {5., 19.30423266812337}]}]}, {
      LineBox[{{6., 28.342043631576193`}, {6., 20.342043631576193`}}], 
      LineBox[{Offset[{1.5, 0}, {6., 28.342043631576193`}], 
        Offset[{-1.5, 0}, {6., 28.342043631576193`}]}], 
      LineBox[{Offset[{1.5, 0}, {6., 20.342043631576193`}], 
        Offset[{-1.5, 0}, {6., 20.342043631576193`}]}]}, {
      LineBox[{{7., 34.559735906829005`}, {7., 26.559735906829}}], 
      LineBox[{Offset[{1.5, 0}, {7., 34.559735906829005`}], 
        Offset[{-1.5, 0}, {7., 34.559735906829005`}]}], 
      LineBox[{Offset[{1.5, 0}, {7., 26.559735906829}], 
        Offset[{-1.5, 0}, {7., 26.559735906829}]}]}, {
      LineBox[{{8., 41.003120190226696`}, {8., 33.003120190226696`}}], 
      LineBox[{Offset[{1.5, 0}, {8., 41.003120190226696`}], 
        Offset[{-1.5, 0}, {8., 41.003120190226696`}]}], 
      LineBox[{Offset[{1.5, 0}, {8., 33.003120190226696`}], 
        Offset[{-1.5, 0}, {8., 33.003120190226696`}]}]}, {
      LineBox[{{9., 33.759160943435305`}, {9., 25.759160943435305`}}], 
      LineBox[{Offset[{1.5, 0}, {9., 33.759160943435305`}], 
        Offset[{-1.5, 0}, {9., 33.759160943435305`}]}], 
      LineBox[{Offset[{1.5, 0}, {9., 25.759160943435305`}], 
        Offset[{-1.5, 0}, {9., 25.759160943435305`}]}]}, {
      LineBox[{{10., 37.96720660074943}, {10., 29.967206600749428`}}], 
      LineBox[{Offset[{1.5, 0}, {10., 37.96720660074943}], 
        Offset[{-1.5, 0}, {10., 37.96720660074943}]}], 
      LineBox[{Offset[{1.5, 0}, {10., 29.967206600749428`}], 
        Offset[{-1.5, 0}, {10., 29.967206600749428`}]}]}, {
      LineBox[{{11., 41.19533282079235}, {11., 33.19533282079235}}], 
      LineBox[{Offset[{1.5, 0}, {11., 41.19533282079235}], 
        Offset[{-1.5, 0}, {11., 41.19533282079235}]}], 
      LineBox[{Offset[{1.5, 0}, {11., 33.19533282079235}], 
        Offset[{-1.5, 0}, {11., 33.19533282079235}]}]}, {
      LineBox[{{12., 40.54910718498606}, {12., 32.54910718498606}}], 
      LineBox[{Offset[{1.5, 0}, {12., 40.54910718498606}], 
        Offset[{-1.5, 0}, {12., 40.54910718498606}]}], 
      LineBox[{Offset[{1.5, 0}, {12., 32.54910718498606}], 
        Offset[{-1.5, 0}, {12., 32.54910718498606}]}]}, {
      LineBox[{{13., 43.80868121211029}, {13., 35.80868121211029}}], 
      LineBox[{Offset[{1.5, 0}, {13., 43.80868121211029}], 
        Offset[{-1.5, 0}, {13., 43.80868121211029}]}], 
      LineBox[{Offset[{1.5, 0}, {13., 35.80868121211029}], 
        Offset[{-1.5, 0}, {13., 35.80868121211029}]}]}, {
      LineBox[{{14., 50.4606138618991}, {14., 42.4606138618991}}], 
      LineBox[{Offset[{1.5, 0}, {14., 50.4606138618991}], 
        Offset[{-1.5, 0}, {14., 50.4606138618991}]}], 
      LineBox[{Offset[{1.5, 0}, {14., 42.4606138618991}], 
        Offset[{-1.5, 0}, {14., 42.4606138618991}]}]}, {
      LineBox[{{15., 56.34288290429642}, {15., 48.34288290429642}}], 
      LineBox[{Offset[{1.5, 0}, {15., 56.34288290429642}], 
        Offset[{-1.5, 0}, {15., 56.34288290429642}]}], 
      LineBox[{Offset[{1.5, 0}, {15., 48.34288290429642}], 
        Offset[{-1.5, 0}, {15., 48.34288290429642}]}]}, {
      LineBox[{{16., 68.03940477039396}, {16., 60.03940477039396}}], 
      LineBox[{Offset[{1.5, 0}, {16., 68.03940477039396}], 
        Offset[{-1.5, 0}, {16., 68.03940477039396}]}], 
      LineBox[{Offset[{1.5, 0}, {16., 60.03940477039396}], 
        Offset[{-1.5, 0}, {16., 60.03940477039396}]}]}, {
      LineBox[{{17., 60.71259432105633}, {17., 52.71259432105633}}], 
      LineBox[{Offset[{1.5, 0}, {17., 60.71259432105633}], 
        Offset[{-1.5, 0}, {17., 60.71259432105633}]}], 
      LineBox[{Offset[{1.5, 0}, {17., 52.71259432105633}], 
        Offset[{-1.5, 0}, {17., 52.71259432105633}]}]}, {
      LineBox[{{18., 65.35965774395106}, {18., 57.359657743951054`}}], 
      LineBox[{Offset[{1.5, 0}, {18., 65.35965774395106}], 
        Offset[{-1.5, 0}, {18., 65.35965774395106}]}], 
      LineBox[{Offset[{1.5, 0}, {18., 57.359657743951054`}], 
        Offset[{-1.5, 0}, {18., 57.359657743951054`}]}]}, {
      LineBox[{{19., 68.34400699858291}, {19., 60.34400699858291}}], 
      LineBox[{Offset[{1.5, 0}, {19., 68.34400699858291}], 
        Offset[{-1.5, 0}, {19., 68.34400699858291}]}], 
      LineBox[{Offset[{1.5, 0}, {19., 60.34400699858291}], 
        Offset[{-1.5, 0}, {19., 60.34400699858291}]}]}, {
      LineBox[{{20., 70.37004437133304}, {20., 62.37004437133304}}], 
      LineBox[{Offset[{1.5, 0}, {20., 70.37004437133304}], 
        Offset[{-1.5, 0}, {20., 70.37004437133304}]}], 
      LineBox[{Offset[{1.5, 0}, {20., 62.37004437133304}], 
        Offset[{-1.5, 0}, {20., 62.37004437133304}]}]}, {
      LineBox[{{21., 74.76260650928123}, {21., 66.76260650928123}}], 
      LineBox[{Offset[{1.5, 0}, {21., 74.76260650928123}], 
        Offset[{-1.5, 0}, {21., 74.76260650928123}]}], 
      LineBox[{Offset[{1.5, 0}, {21., 66.76260650928123}], 
        Offset[{-1.5, 0}, {21., 66.76260650928123}]}]}, {
      LineBox[{{22., 78.18515639968481}, {22., 70.18515639968481}}], 
      LineBox[{Offset[{1.5, 0}, {22., 78.18515639968481}], 
        Offset[{-1.5, 0}, {22., 78.18515639968481}]}], 
      LineBox[{Offset[{1.5, 0}, {22., 70.18515639968481}], 
        Offset[{-1.5, 0}, {22., 70.18515639968481}]}]}, {
      LineBox[{{23., 80.19065754298724}, {23., 72.19065754298724}}], 
      LineBox[{Offset[{1.5, 0}, {23., 80.19065754298724}], 
        Offset[{-1.5, 0}, {23., 80.19065754298724}]}], 
      LineBox[{Offset[{1.5, 0}, {23., 72.19065754298724}], 
        Offset[{-1.5, 0}, {23., 72.19065754298724}]}]}, {
      LineBox[{{24., 75.34243852573245}, {24., 67.34243852573245}}], 
      LineBox[{Offset[{1.5, 0}, {24., 75.34243852573245}], 
        Offset[{-1.5, 0}, {24., 75.34243852573245}]}], 
      LineBox[{Offset[{1.5, 0}, {24., 67.34243852573245}], 
        Offset[{-1.5, 0}, {24., 67.34243852573245}]}]}, {
      LineBox[{{25., 85.68005896422669}, {25., 77.68005896422669}}], 
      LineBox[{Offset[{1.5, 0}, {25., 85.68005896422669}], 
        Offset[{-1.5, 0}, {25., 85.68005896422669}]}], 
      LineBox[{Offset[{1.5, 0}, {25., 77.68005896422669}], 
        Offset[{-1.5, 0}, {25., 77.68005896422669}]}]}, {
      LineBox[{{26., 86.69882502470743}, {26., 78.69882502470743}}], 
      LineBox[{Offset[{1.5, 0}, {26., 86.69882502470743}], 
        Offset[{-1.5, 0}, {26., 86.69882502470743}]}], 
      LineBox[{Offset[{1.5, 0}, {26., 78.69882502470743}], 
        Offset[{-1.5, 0}, {26., 78.69882502470743}]}]}, {
      LineBox[{{27., 84.54370338356986}, {27., 76.54370338356986}}], 
      LineBox[{Offset[{1.5, 0}, {27., 84.54370338356986}], 
        Offset[{-1.5, 0}, {27., 84.54370338356986}]}], 
      LineBox[{Offset[{1.5, 0}, {27., 76.54370338356986}], 
        Offset[{-1.5, 0}, {27., 76.54370338356986}]}]}, {
      LineBox[{{28., 92.12872836797479}, {28., 84.12872836797479}}], 
      LineBox[{Offset[{1.5, 0}, {28., 92.12872836797479}], 
        Offset[{-1.5, 0}, {28., 92.12872836797479}]}], 
      LineBox[{Offset[{1.5, 0}, {28., 84.12872836797479}], 
        Offset[{-1.5, 0}, {28., 84.12872836797479}]}]}, {
      LineBox[{{29., 92.02863833456037}, {29., 84.02863833456037}}], 
      LineBox[{Offset[{1.5, 0}, {29., 92.02863833456037}], 
        Offset[{-1.5, 0}, {29., 92.02863833456037}]}], 
      LineBox[{Offset[{1.5, 0}, {29., 84.02863833456037}], 
        Offset[{-1.5, 0}, {29., 84.02863833456037}]}]}, {
      LineBox[{{30., 104.20600557692805`}, {30., 96.20600557692805}}], 
      LineBox[{Offset[{1.5, 0}, {30., 104.20600557692805`}], 
        Offset[{-1.5, 0}, {30., 104.20600557692805`}]}], 
      LineBox[{Offset[{1.5, 0}, {30., 96.20600557692805}], 
        Offset[{-1.5, 0}, {30., 96.20600557692805}]}]}}}, {}},
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  Axes->True,
  AxesOrigin->{0, 0},
  PlotRange->{{0., 30.}, {0., 100.20600557692805`}},
  PlotRangeClipping->True,
  PlotRangePadding->{
    Scaled[0.02], 
    Scaled[0.02]}]], "Output",
 CellChangeTimes->{
  3.4005848638859997`*^9, {3.400584907835*^9, 3.400584931013*^9}, 
   3.400585012458*^9, {3.400585159389*^9, 3.40058516317*^9}, {
   3.400585208304*^9, 3.400585230116*^9}, 3.416688820013*^9, 
   3.416689019019*^9, 3.416689056269*^9, 3.443184204095348*^9, 
   3.4431845597979145`*^9, 3.443185250179946*^9, 3.5250740807697897`*^9, 
   3.5250969298393927`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"fit", "=", 
  RowBox[{"LinearModelFit", "[", 
   RowBox[{"linData", ",", 
    RowBox[{"{", 
     RowBox[{"1", ",", " ", "x"}], "}"}], ",", "x"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.400585232391*^9, 3.400585233575*^9}, {
  3.4431845451644516`*^9, 3.443184547210656*^9}}],

Cell[BoxData[
 RowBox[{"FittedModel", "[", 
  PanelBox[
   TagBox[
    RowBox[{"6.616393613284181`", "\[VeryThinSpace]", "+", 
     RowBox[{"2.9473607874621917`", " ", "x"}]}],
    Short],
   FrameMargins->5], "]"}]], "Output",
 CellChangeTimes->{
  3.4005849343900003`*^9, 3.400585013288*^9, {3.400585159959*^9, 
   3.400585164369*^9}, {3.400585209455*^9, 3.4005852339379997`*^9}, 
   3.416689020334*^9, 3.416689058578*^9, {3.44318455345228*^9, 
   3.443184560920027*^9}, 3.44318525162409*^9, 3.525074083789794*^9, 
   3.525096931679395*^9}]
}, Open  ]],

Cell[TextData[{
 "This FittedModel has a lot of information hidden inside it. Take a look at \
its help page for more details. We can extract what it calls \"Standard Error\
\"s on these parameters. Note, however, that we haven't yet let \
LinearModelFit know about our error bars. That means that ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " must be making some assumption to generate these errors. Below, we'll \
compare these standard errors to those that we would calculate by hand. "
}], "Text",
 CellChangeTimes->{{3.443184727159649*^9, 3.44318480447338*^9}, {
  3.4431887569775906`*^9, 3.4431887744373364`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"fit", "[", "\"\<ParameterErrors\>\"", "]"}]], "Input",
 CellChangeTimes->{{3.4431846118401184`*^9, 3.4431846120731416`*^9}, {
  3.4431846871186457`*^9, 3.4431847206329966`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"1.4250329562131172`", ",", "0.08027027835627219`"}], 
  "}"}]], "Output",
 CellChangeTimes->{
  3.443184612710205*^9, {3.443184692288162*^9, 3.4431847212820616`*^9}, 
   3.4431848158805203`*^9, 3.443185254453373*^9, 3.5250741058698254`*^9, 
   3.5250969328893967`*^9}]
}, Open  ]],

Cell[TextData[{
 StyleBox["Quick exercise\n", "Subsection"],
 "\nMake up some polynomial data with noise. Fit it with LinearModelFit."
}], "Text",
 CellChangeTimes->{{3.443184779941927*^9, 3.4431848287718096`*^9}}]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Solution", "Subsubtitle"],

Cell[CellGroupData[{

Cell["Linear weighted fit", "Subsubsection"],

Cell[TextData[{
 "To get really useful errors in the fit parameters we need to let ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " know what our error bars are. We do this by performing a weighted fit. The \
larger a point's error bar, the smaller the weight we'll give it in the fit. \
LinearModelFit's documentation is a bit vague as to what it will actually do, \
but it says that:\n\n \" Using ",
 Cell[BoxData[
  RowBox[{
   ButtonBox["VarianceEstimatorFunction",
    BaseStyle->"Link",
    ButtonData->"paclet:ref/VarianceEstimatorFunction"], "->", 
   RowBox[{"(", 
    RowBox[{"1", "&"}], ")"}]}]], "InlineFormula"],
 " and ",
 Cell[BoxData[
  RowBox[{
   ButtonBox["Weights",
    BaseStyle->"Link",
    ButtonData->"paclet:ref/Weights"], "->", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"1", "/", 
      SuperscriptBox[
       SubscriptBox[
        StyleBox[
         StyleBox[
          RowBox[{
           StyleBox["\[CapitalDelta]",
            FontSlant->"Plain"], "y"}]], "TI"], 
        StyleBox["1", "TR"]], "2"]}], ",", 
     RowBox[{"1", "/", 
      SuperscriptBox[
       SubscriptBox[
        StyleBox[
         StyleBox[
          RowBox[{
           StyleBox["\[CapitalDelta]",
            FontSlant->"Plain"], "y"}]], "TI"], 
        StyleBox["2", "TR"]], "2"]}], ",", 
     StyleBox["\[Ellipsis]", "TR"]}], 
    StyleBox["}", "TR"]}]}]], "InlineFormula"],
 ", ",
 Cell[BoxData[
  SubscriptBox[
   StyleBox[
    StyleBox[
     RowBox[{
      StyleBox["\[CapitalDelta]",
       FontSlant->"Plain"], "y"}]], "TI"], 
   StyleBox["i", "TI"]]], "InlineFormula"],
 " is treated as the known uncertainty of measurement ",
 Cell[BoxData[
  SubscriptBox[
   StyleBox["y", "TI"], 
   StyleBox["i", "TI"]]], "InlineFormula"],
 " and parameter standard errors are effectively computed only from the \
weights.\"\n \nSo let's try that. Re-running the fit with these weights \
(remember that I generated the data with noise on the y-axis of standard \
deviation 4)."
}], "Text",
 CellChangeTimes->{{3.4431849008570175`*^9, 3.443184939037835*^9}, {
  3.443185044623392*^9, 3.4431850718591156`*^9}, {3.443188790102903*^9, 
  3.443188887624654*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"fit", "=", 
   RowBox[{"LinearModelFit", "[", 
    RowBox[{"linData", ",", 
     RowBox[{"{", 
      RowBox[{"1", ",", "x"}], "}"}], ",", "x", ",", 
     RowBox[{"VarianceEstimatorFunction", "\[Rule]", 
      RowBox[{"(", 
       RowBox[{"1", "&"}], ")"}]}], ",", 
     RowBox[{"Weights", "\[Rule]", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         FractionBox["1", 
          SuperscriptBox["4", "2"]], "&"}], "/@", "linData"}], ")"}]}]}], 
    "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"fit", "[", "\"\<BestFit\>\"", "]"}], "\[IndentingNewLine]", 
 RowBox[{"fit", "[", "\"\<ParameterErrors\>\"", "]"}]}], "Input",
 CellChangeTimes->{
  3.400585181874*^9, {3.400585213474*^9, 3.400585244995*^9}, {
   3.4431849616941004`*^9, 3.443184983653296*^9}, {3.443185077719702*^9, 
   3.44318510520745*^9}, {3.443185176992628*^9, 3.4431851880917377`*^9}, {
   3.4431852410750356`*^9, 3.443185261471075*^9}}],

Cell[BoxData[
 RowBox[{"6.616393613284157`", "\[VeryThinSpace]", "+", 
  RowBox[{"2.9473607874621917`", " ", "x"}]}]], "Output",
 CellChangeTimes->{
  3.400585038586*^9, 3.400585087163*^9, {3.400585168745*^9, 
   3.400585182368*^9}, 3.400585214043*^9, 3.400585246389*^9, 
   3.4166890613929996`*^9, {3.4431849656484957`*^9, 3.4431849839163227`*^9}, {
   3.4431850920311327`*^9, 3.443185107798709*^9}, {3.443185177952724*^9, 
   3.4431851889308214`*^9}, {3.4431852418541136`*^9, 3.4431852617751055`*^9}, 
   3.5250741854699364`*^9, 3.525096938129404*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"1.4978912380141443`", ",", "0.08437429190573395`"}], 
  "}"}]], "Output",
 CellChangeTimes->{
  3.400585038586*^9, 3.400585087163*^9, {3.400585168745*^9, 
   3.400585182368*^9}, 3.400585214043*^9, 3.400585246389*^9, 
   3.4166890613929996`*^9, {3.4431849656484957`*^9, 3.4431849839163227`*^9}, {
   3.4431850920311327`*^9, 3.443185107798709*^9}, {3.443185177952724*^9, 
   3.4431851889308214`*^9}, {3.4431852418541136`*^9, 3.4431852617751055`*^9}, 
   3.5250741854699364`*^9, 3.525096938129404*^9}]
}, Open  ]],

Cell[TextData[{
 "How do these estimates compare to the analytic results for the \
uncertainties in the fit parameters? (You can find the following formulae and \
their derivation in Numerical Recipes, section 15.2, available free online ",
 ButtonBox["here",
  BaseStyle->"Hyperlink",
  ButtonData:>{
    URL["http://www.nrbook.com/a/bookcpdf.html"], None}],
 ")."
}], "Text",
 CellChangeTimes->{3.4431852775236797`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"linearFitCombs", "[", "dat_", "]"}], ":=", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"s", "\[Rule]", 
     RowBox[{"Plus", "@@", 
      FractionBox["1", 
       RowBox[{
        RowBox[{
         SuperscriptBox[
          RowBox[{"#", "\[LeftDoubleBracket]", "3", "\[RightDoubleBracket]"}],
           "2"], "&"}], "/@", "dat"}]]}]}], ",", 
    RowBox[{"sx", "\[Rule]", 
     RowBox[{"Plus", "@@", 
      FractionBox[
       RowBox[{
        RowBox[{
         RowBox[{"#", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}], 
         "&"}], "/@", "dat"}], 
       RowBox[{
        RowBox[{
         SuperscriptBox[
          RowBox[{"#", "\[LeftDoubleBracket]", "3", "\[RightDoubleBracket]"}],
           "2"], "&"}], "/@", "dat"}]]}]}], ",", 
    RowBox[{"sy", "\[Rule]", 
     RowBox[{"Plus", "@@", 
      FractionBox[
       RowBox[{
        RowBox[{
         RowBox[{"#", "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}], 
         "&"}], "/@", "dat"}], 
       RowBox[{
        RowBox[{
         SuperscriptBox[
          RowBox[{"#", "\[LeftDoubleBracket]", "3", "\[RightDoubleBracket]"}],
           "2"], "&"}], "/@", "dat"}]]}]}], ",", 
    RowBox[{"sxx", "\[Rule]", 
     RowBox[{"Plus", "@@", 
      FractionBox[
       RowBox[{
        RowBox[{
         SuperscriptBox[
          RowBox[{"#", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}],
           "2"], "&"}], "/@", "dat"}], 
       RowBox[{
        RowBox[{
         SuperscriptBox[
          RowBox[{"#", "\[LeftDoubleBracket]", "3", "\[RightDoubleBracket]"}],
           "2"], "&"}], "/@", "dat"}]]}]}], ",", 
    RowBox[{"sxy", "->", 
     RowBox[{"Plus", "@@", 
      FractionBox[
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"#", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}],
           " ", 
          RowBox[{
          "#", "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}]}], 
         "&"}], "/@", "dat"}], 
       RowBox[{
        RowBox[{
         SuperscriptBox[
          RowBox[{"#", "\[LeftDoubleBracket]", "3", "\[RightDoubleBracket]"}],
           "2"], "&"}], "/@", "dat"}]]}]}]}], "}"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"linearFit", "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"\[CapitalDelta]", "\[Rule]", 
      RowBox[{
       RowBox[{"s", " ", "sxx"}], "-", 
       SuperscriptBox["sx", "2"]}]}], ",", 
     RowBox[{"a", "\[Rule]", 
      FractionBox[
       RowBox[{
        RowBox[{"sxx", " ", "sy"}], "-", 
        RowBox[{"sx", " ", "sxy"}]}], "\[CapitalDelta]"]}], ",", 
     RowBox[{"b", "\[Rule]", 
      FractionBox[
       RowBox[{
        RowBox[{"s", " ", "sxy"}], " ", "-", 
        RowBox[{"sx", " ", "sy"}]}], "\[CapitalDelta]"]}], ",", 
     RowBox[{
      SubscriptBox["\[Sigma]", "a"], "->", 
      SqrtBox[
       FractionBox["sxx", "\[CapitalDelta]"]]}], ",", 
     RowBox[{
      SubscriptBox["\[Sigma]", "b"], "->", 
      SqrtBox[
       FractionBox["s", "\[CapitalDelta]"]]}]}], "}"}]}], ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"{", 
     RowBox[{"a", ",", "b", ",", 
      SubscriptBox["\[Sigma]", "a"], ",", 
      SubscriptBox["\[Sigma]", "b"]}], "}"}], "//.", "linearFit"}], "/.", 
   RowBox[{"linearFitCombs", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"Join", "[", 
       RowBox[{"#", ",", 
        RowBox[{"{", "4", "}"}]}], "]"}], "&"}], "/@", "linData"}], "]"}]}], "//",
   "N"}]], "Input",
 CellChangeTimes->{3.4005852630369997`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "6.616393613284181`", ",", "2.947360787462192`", ",", "1.4978912380141425`",
    ",", "0.08437429190573391`"}], "}"}]], "Output",
 CellChangeTimes->{3.4005852638129997`*^9, 3.4166890633599997`*^9, 
  3.443185120368966*^9, 3.5250742028709607`*^9, 3.525096940649408*^9}]
}, Open  ]],

Cell[TextData[{
 "They're the same, which is reassuring! Interestingly they're similar to the \
values ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " produced without knowing about the error bars, but not the same. Once \
again though, I stress that ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " is making assumptions when deriving these errors without point error bars. \
We'll see how one of these assumptions could break down right now."
}], "Text",
 CellChangeTimes->{{3.443188899416833*^9, 3.4431889963705273`*^9}, {
  3.443189048424732*^9, 3.443189059965886*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Different weights", "Subsubsection"],

Cell["\<\
What about if the data don't all have the same weight? First of all neglect \
the error bars, and then try fitting a line using LinearModelFit and the \
analytic formulae above for data that have unequally weighted points. How do \
the fits and errors compare?\
\>", "Text",
 CellChangeTimes->{{3.443189063354225*^9, 3.4431890694398336`*^9}, {
  3.4431893533262196`*^9, 3.4431893915090375`*^9}}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Solution", "Subsubtitle"],

Cell[CellGroupData[{

Cell["Playtime", "Section"],

Cell["\<\
Once again, here are some exercises that you can have a go at if you're \
feeling keen. They expand upon a few of the points we've covered above. \
\>", "Text"],

Cell[CellGroupData[{

Cell["Do it yourself", "Subsubsection"],

Cell[TextData[{
 "Try doing a non-linear fit yourself by minimising ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["\[Chi]", "2"], TraditionalForm]]],
 " explicitly. Solve for fit parameters that give a ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["\[Chi]", "2"], "="}], TraditionalForm]]],
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox[
     SubscriptBox["\[Chi]", "min"], "2"], "+", "1"}], TraditionalForm]]],
 ", where ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    SubscriptBox["\[Chi]", "min"], "2"], TraditionalForm]]],
 " is the value of ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["\[Chi]", 
    RowBox[{"2", " "}]], TraditionalForm]]],
 "at the minimum (you'll want to vary them one at a time leaving the others \
fixed). This should give you estimates for the errors in the fitted \
parameters. Try and compare these to ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "'s parameter errors."
}], "Text",
 CellChangeTimes->{
  3.443189878012683*^9, {3.44319000559044*^9, 3.4431900093568163`*^9}, {
   3.4431901326941485`*^9, 3.443190133727252*^9}}]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Solution", "Subsubtitle"],

Cell[CellGroupData[{

Cell["Bootsrapping the dataset", "Subsubsection"],

Cell["\<\
A very simple and general technique for estimating the uncertainty in any \
statistical estimator is the bootstrap (here the fitting function is the \
estimator - it estimates the coefficients in the fitted model). For now let's \
not worry too much about the details of how the bootstrap works: it's easier \
to do than to explain! The basic idea is to generate fake data sets from the \
real data set. You do this by pulling out points from the real data set at \
random until you have as many points in the fake data set as in the real one. \
You don't worry if you pull out the same point more than once. You then apply \
the estimator to the fake data set (in this case, fit it). Do this a few \
thousand times and you get a histogram of likely estimated values. Take the \
standard deviation of this and you've got an estimate for the error in the \
estimator! Easy, right?

To get you started, here's a function that generates a new data set (often \
called a replicate) from an original data set.\
\>", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"makeBootstrapReplicate", "[", "data_", "]"}], ":=", 
  RowBox[{"data", "\[LeftDoubleBracket]", 
   RowBox[{"RandomInteger", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"1", ",", 
       RowBox[{"Length", "[", "data", "]"}]}], "}"}], ",", 
     RowBox[{"Length", "[", "data", "]"}]}], "]"}], 
   "\[RightDoubleBracket]"}]}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"makeBootstrapReplicate", "[", "linData", "]"}]], "Input",
 CellChangeTimes->{3.4005852900150003`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"30", ",", "100.20600557692805`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"13", ",", "39.80868121211029`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"2", ",", "13.94988723398587`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"19", ",", "64.34400699858291`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"28", ",", "88.12872836797479`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"18", ",", "61.359657743951054`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"21", ",", "70.76260650928123`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"10", ",", "33.96720660074943`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"11", ",", "37.19533282079235`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"25", ",", "81.68005896422669`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"16", ",", "64.03940477039396`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"15", ",", "52.34288290429642`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"5", ",", "23.30423266812337`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"7", ",", "30.559735906829`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"4", ",", "15.865793510932651`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"20", ",", "66.37004437133304`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"13", ",", "39.80868121211029`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"11", ",", "37.19533282079235`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"23", ",", "76.19065754298724`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"22", ",", "74.18515639968481`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"27", ",", "80.54370338356986`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"17", ",", "56.71259432105633`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"19", ",", "64.34400699858291`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"27", ",", "80.54370338356986`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"3", ",", "15.532565167297973`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"9", ",", "29.759160943435305`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"2", ",", "13.94988723398587`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"25", ",", "81.68005896422669`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"10", ",", "33.96720660074943`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"22", ",", "74.18515639968481`"}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{3.4005852908970003`*^9, 3.416689064856*^9, 
  3.4431901604099197`*^9, 3.5250970815196223`*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Fitting a replicate", "Subsubsection"],

Cell[TextData[{
 "Write a function that generates a bootstrap replicate and fits it. The \
function should return the fit parameters.\n",
 StyleBox["Hint:",
  FontWeight->"Bold"],
 " because you're not interested in errors, just the fit parameters, you can \
use FindFit if you'd rather."
}], "Text"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Solution", "Subsubtitle"],

Cell[CellGroupData[{

Cell["Bootstrap estimate of parameter uncertainties", "Subsubsection"],

Cell[TextData[{
 "Build a table of fit results using the function you've just written. ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["10", "4"], TraditionalForm]]],
 " fits should be plenty. Histogram the fitted parameter estimates. Take the \
standard deviation of the fitted parameter estimates to estimate the error in \
the fitted paramters. Compare these errors to those given above.\n",
 StyleBox["Hint: ",
  FontWeight->"Bold"],
 "there's a function Histogram and a function StandardDeviation."
}], "Text"]
}, Open  ]]
}, Open  ]],

Cell["Solution", "Subsubtitle"]
}, Open  ]]
},
WindowSize->{809, 636},
WindowMargins->{{80, Automatic}, {Automatic, 19}},
FrontEndVersion->"8.0 for Microsoft Windows (64-bit) (February 23, 2011)",
StyleDefinitions->FrontEnd`FileName[{"Creative"}, "PastelColor.nb", 
  CharacterEncoding -> "WindowsANSI"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[567, 22, 158, 2, 65, "Title"],
Cell[728, 26, 148, 2, 23, "Subtitle"],
Cell[879, 30, 669, 14, 99, "Text"],
Cell[1551, 46, 1079, 21, 173, "Text"],
Cell[CellGroupData[{
Cell[2655, 71, 39, 0, 75, "Section"],
Cell[2697, 73, 681, 16, 119, "Text"],
Cell[3381, 91, 977, 30, 62, "Input"],
Cell[4361, 123, 46, 0, 29, "Text"],
Cell[CellGroupData[{
Cell[4432, 127, 353, 9, 62, "Input"],
Cell[4788, 138, 922, 20, 213, "Output"]
}, Open  ]],
Cell[5725, 161, 542, 10, 101, "Text"],
Cell[6270, 173, 334, 10, 43, "Input"],
Cell[CellGroupData[{
Cell[6629, 187, 491, 14, 43, "Input"],
Cell[7123, 203, 514, 9, 40, "Output"]
}, Open  ]],
Cell[7652, 215, 355, 8, 47, "Text"],
Cell[CellGroupData[{
Cell[8032, 227, 126, 2, 41, "Input"],
Cell[8161, 231, 388, 10, 43, "Output"]
}, Open  ]],
Cell[8564, 244, 226, 4, 29, "Text"],
Cell[CellGroupData[{
Cell[8815, 252, 432, 10, 102, "Input"],
Cell[9250, 264, 2661, 50, 213, "Output"]
}, Open  ]],
Cell[11926, 317, 230, 4, 47, "Text"],
Cell[CellGroupData[{
Cell[12181, 325, 163, 5, 41, "Input"],
Cell[12347, 332, 287, 4, 40, "Output"]
}, Open  ]],
Cell[12649, 339, 207, 4, 47, "Text"],
Cell[CellGroupData[{
Cell[12881, 347, 31, 0, 36, "Subsection"],
Cell[12915, 349, 240, 4, 47, "Text"],
Cell[CellGroupData[{
Cell[13180, 357, 49, 0, 27, "Subsubsection"],
Cell[13232, 359, 1136, 29, 137, "Text"]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[14429, 395, 31, 0, 25, "Subsubtitle"],
Cell[CellGroupData[{
Cell[14485, 399, 49, 0, 27, "Subsubsection"],
Cell[14537, 401, 4956, 158, 222, "Input"],
Cell[19496, 561, 1122, 33, 121, "Text"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[20667, 600, 31, 0, 25, "Subsubtitle"],
Cell[CellGroupData[{
Cell[20723, 604, 28, 0, 75, "Section"],
Cell[20754, 606, 504, 9, 119, "Text"],
Cell[CellGroupData[{
Cell[21283, 619, 31, 0, 36, "Subsection"],
Cell[21317, 621, 456, 9, 65, "Text"],
Cell[21776, 632, 394, 9, 82, "Input"],
Cell[CellGroupData[{
Cell[22195, 645, 42, 0, 27, "Subsubsection"],
Cell[22240, 647, 1822, 46, 191, "Text"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[24135, 701, 31, 0, 25, "Subsubtitle"],
Cell[CellGroupData[{
Cell[24191, 705, 29, 0, 75, "Section"],
Cell[24223, 707, 2998, 44, 479, "Text"],
Cell[CellGroupData[{
Cell[27246, 755, 31, 0, 36, "Subsection"],
Cell[27280, 757, 321, 5, 65, "Text"],
Cell[CellGroupData[{
Cell[27626, 766, 53, 0, 27, "Subsubsection"],
Cell[27682, 768, 764, 13, 155, "Text"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[28519, 789, 31, 0, 25, "Subsubtitle"],
Cell[CellGroupData[{
Cell[28575, 793, 56, 0, 27, "Subsubsection"],
Cell[28634, 795, 486, 16, 65, "Text"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[29169, 817, 31, 0, 25, "Subsubtitle"],
Cell[CellGroupData[{
Cell[29225, 821, 41, 0, 27, "Subsubsection"],
Cell[29269, 823, 724, 17, 83, "Text"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[30042, 846, 31, 0, 25, "Subsubtitle"],
Cell[CellGroupData[{
Cell[30098, 850, 46, 0, 27, "Subsubsection"],
Cell[30147, 852, 563, 12, 83, "Text"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[30759, 870, 31, 0, 25, "Subsubtitle"],
Cell[CellGroupData[{
Cell[30815, 874, 40, 0, 75, "Section"],
Cell[30858, 876, 2886, 43, 445, "Text"],
Cell[CellGroupData[{
Cell[33769, 923, 120, 1, 27, "Subsubsection"],
Cell[33892, 926, 501, 9, 65, "Text"],
Cell[34396, 937, 136, 2, 41, "Input"],
Cell[CellGroupData[{
Cell[34557, 943, 833, 25, 62, "Input"],
Cell[35393, 970, 10326, 175, 253, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[45756, 1150, 299, 7, 41, "Input"],
Cell[46058, 1159, 542, 13, 60, "Output"]
}, Open  ]],
Cell[46615, 1175, 628, 11, 83, "Text"],
Cell[CellGroupData[{
Cell[47268, 1190, 201, 3, 41, "Input"],
Cell[47472, 1195, 308, 7, 40, "Output"]
}, Open  ]],
Cell[47795, 1205, 214, 4, 66, "Text"]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[48070, 1216, 31, 0, 25, "Subsubtitle"],
Cell[CellGroupData[{
Cell[48126, 1220, 44, 0, 27, "Subsubsection"],
Cell[48173, 1222, 2163, 64, 176, "Text"],
Cell[CellGroupData[{
Cell[50361, 1290, 955, 23, 126, "Input"],
Cell[51319, 1315, 553, 9, 40, "Output"],
Cell[51875, 1326, 538, 10, 40, "Output"]
}, Open  ]],
Cell[52428, 1339, 421, 10, 47, "Text"],
Cell[52852, 1351, 2179, 65, 102, "Input"],
Cell[55034, 1418, 850, 26, 64, "Input"],
Cell[CellGroupData[{
Cell[55909, 1448, 479, 15, 41, "Input"],
Cell[56391, 1465, 310, 6, 40, "Output"]
}, Open  ]],
Cell[56716, 1474, 586, 13, 65, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[57339, 1492, 42, 0, 27, "Subsubsection"],
Cell[57384, 1494, 405, 7, 65, "Text"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[57838, 1507, 31, 0, 25, "Subsubtitle"],
Cell[CellGroupData[{
Cell[57894, 1511, 27, 0, 75, "Section"],
Cell[57924, 1513, 170, 3, 47, "Text"],
Cell[CellGroupData[{
Cell[58119, 1520, 39, 0, 27, "Subsubsection"],
Cell[58161, 1522, 1095, 34, 69, "Text"]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[59317, 1563, 31, 0, 25, "Subsubtitle"],
Cell[CellGroupData[{
Cell[59373, 1567, 49, 0, 27, "Subsubsection"],
Cell[59425, 1569, 1028, 16, 173, "Text"],
Cell[60456, 1587, 376, 10, 41, "Input"],
Cell[CellGroupData[{
Cell[60857, 1601, 125, 2, 41, "Input"],
Cell[60985, 1605, 2473, 64, 117, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[63507, 1675, 44, 0, 27, "Subsubsection"],
Cell[63554, 1677, 300, 7, 47, "Text"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[63903, 1690, 31, 0, 25, "Subsubtitle"],
Cell[CellGroupData[{
Cell[63959, 1694, 70, 0, 27, "Subsubsection"],
Cell[64032, 1696, 514, 11, 85, "Text"]
}, Open  ]]
}, Open  ]],
Cell[64573, 1711, 31, 0, 25, "Subsubtitle"]
}, Open  ]]
}
]
*)

(* End of internal cache information *)
